; Quick test for Jury
;
; Showing a 640*200 image in lowres without using $ffff820f
;
; A0/A1 locked
; D0 is trashed every scanline that has visible graphics
;
; 267 scanlines CPU free (85.3%)
; 
; Further optimizations possible by aligning image to 64k boundary,
; but that also limits the size of the playfield
;
; Builds with Devpac 3 / gen.ttp
;
; 2018-01-13
; ae@dhs.nu


		output	.tos


		opt O-

		section	text

start:		pea	demo				;Run demo in supervisor
		move.w	#$26,-(sp)			;supexec()
		trap	#14				;
		addq.l	#6,sp				;

		clr.w	-(sp)				;pterm()
		trap	#1				;



demo:		bsr	save_and_setup_hardware
		bsr	copy_image_to_screen
		bsr	generate_displaylist
		bsr	save_mfp
		bsr	setup_mfp


;--------------- MAINLOOP ------------------------------		
.mainloop:	tst.w	vblcount			;Wait VBL
		beq.s	.mainloop
		clr.w	vblcount

		;test how many free scanlines we got
		ifne	0
		move.w	#267-1,d7
.cpu1scanline:	dcb.w	128-3,$4e71
		dbra	d7,.cpu1scanline
		not.w	$ffff8240.w
		endc

		cmp.b	#$39,$fffffc02.w		;Space?
		bne	.mainloop

		bsr	restore
		rts


;-------------- VBL --------------------------------------
vbl:		movem.l	d0-a6,-(sp)

		addq.w	#1,vblcount

		move.l	scraddr,d0			;Set screen
		lsr.w	#8,d0
		move.l	d0,$ffff8200.w

		movem.l	image,d0-d7			;Set palette
		movem.l	d0-d7,$ffff8240.w

		move.w	#$2700,sr
		clr.b	$fffffa1b.w			;Timer B control (stop)
		bset	#0,$fffffa07.w			;Interrupt enable A (Timer B)
		bset	#0,$fffffa13.w			;Interrupt mask A (Timer B)
		bclr	#3,$fffffa17.w			;Automatic end of interrupt
		move.b	#1,$fffffa21.w			;Timer B data (number of scanlines to next interrupt)
		move.b	#8,$fffffa1b.w			;Timer B control (event mode (HBL))

		movem.l	(sp)+,d0-a6

		;Setup address registers for Timer B HBL and set first displaylist address
		lea	$ffff8203.w,a0
		lea	displaylist,a1

		move.l	(a1)+,d0			;Fetch from displaylist
		movep.l	d0,0(a0)			;Set screen
dummy:		rte


;-------------- TIMER B HBL ----------------------------
timer_b:	move.l	(a1)+,d0			;Fetch from displaylist
		movep.l	d0,0(a0)			;Set screen
		rte



;-------------- MISC -----------------------------------
copy_image_to_screen:
		move.l	scraddr,a0
		lea	image+32,a1
		move.w	#640*200/2/4-1,d7
.loop:		move.l	(a1)+,(a0)+
		dbra	d7,.loop
		rts

generate_displaylist:
		lea	displaylist,a0
		move.l	scraddr,a1
		lea	80(a1),a1			;center
		move.w	#200-1,d7
.loop:		move.l	a1,(a0)+
		lea	320(a1),a1
		dbra	d7,.loop
		rts

save_and_setup_hardware:
		lea	save_screenadr,a0		;Save old screen address
		move.b	$ffff8201.w,(a0)+		;
		move.b	$ffff8203.w,(a0)+		;
		move.b	$ffff820d.w,(a0)+		;

		movem.l	$ffff8240.w,d0-d7		;Save old palette
		movem.l	d0-d7,save_pal			;

		move.l	#screen+256,d0			;Align new screen address
		clr.b	d0				;
		move.l	d0,scraddr			;

		lsr.w	#8,d0				;Set new screen address
		move.l	d0,$ffff8200.w			;

		move.b	$ffff8260.w,save_res		;Save old resolution
		clr.b	$ffff8260.w			;Set low resolution

		move.b	$ffff820a.w,save_freq		;Save old refresh
		move.b	#2,$ffff820a.w			;Set 50 Hz

		move.b	$ffff8265.w,save_hscrl		;Save old hscroll
		move.b	$ffff820f.w,save_lw		;Save old linewidth

		move.b	#$12,$fffffc02.w		;Kill mouse
		rts

save_mfp:		
		move.w	#$2700,sr
		lea	save_vect_mfp,a0
		move.l	$70.w,(a0)+			;Save old VBL
		move.l	$68.w,(a0)+			;Save old HBL
		move.l	$134.w,(a0)+			;Save old Timer A
		move.l	$120.w,(a0)+			;Save old Timer B
		move.l	$114.w,(a0)+			;Save old Timer C
		move.l	$110.w,(a0)+			;Save old Timer D
		move.l	$118.w,(a0)+			;Save old ACIA
		move.b	$fffffa07.w,(a0)+
		move.b	$fffffa09.w,(a0)+
		move.b	$fffffa13.w,(a0)+
		move.b	$fffffa15.w,(a0)+
		move.b	$fffffa19.w,(a0)+
		move.b	$fffffa1b.w,(a0)+
		move.b	$fffffa1f.w,(a0)+
		move.b	$fffffa21.w,(a0)+
		clr.b	$fffffa07.w			;Interrupt enable A (Timer-A & B)
		clr.b	$fffffa13.w			;Interrupt mask A (Timer-A & B)
		clr.b	$fffffa09.w			;Interrupt enable B (Timer-C & D)
		clr.b	$fffffa15.w			;Interrupt mask B (Timer-C & D)
		move.w	#$2300,sr
		rts

setup_mfp:
		move.w	#$2700,sr
		lea	dummy(pc),a0
		move.l	#vbl,$70.w			;Install our own VBL
		move.l	a0,$68.w			;Install our own HBL
		move.l	a0,$134.w			;Install our own Timer A
		move.l	#timer_b,$120.w			;Install our own Timer B
		move.l	a0,$114.w			;Install our own Timer C
		move.l	a0,$110.w			;Install our own Timer D
		move.l	a0,$118.w			;Install our own ACIA
		move.w	#$2300,sr
		rts
restore:
		move.w	#$2700,sr			;Stop all interrupts
		lea	save_vect_mfp,a0
		move.l	(a0)+,$70.w			;Restore old VBL
		move.l	(a0)+,$68.w			;Restore old HBL
		move.l	(a0)+,$134.w			;Restore old Timer A
		move.l	(a0)+,$120.w			;Restore old Timer B
		move.l	(a0)+,$114.w			;Restore old Timer C
		move.l	(a0)+,$110.w			;Restore old Timer D
		move.l	(a0)+,$118.w			;Restore old ACIA
		move.b	(a0)+,$fffffa07.w
		move.b	(a0)+,$fffffa09.w
		move.b	(a0)+,$fffffa13.w
		move.b	(a0)+,$fffffa15.w
		move.b	(a0)+,$fffffa19.w
		move.b	(a0)+,$fffffa1b.w
		move.b	(a0)+,$fffffa1f.w
		move.b	(a0)+,$fffffa21.w
		move.w	#$2300,sr			;Interrupts back on

		move.b	save_res,$ffff8260.w		;Restore old resolution
		move.b	save_freq,$ffff820a.w		;Restore old refresh

		move.b	save_hscrl,$ffff8265.w		;Restore old hscroll
		move.b	save_lw,$ffff820f.w		;Restore old linewidth

		movem.l	save_pal,d0-d7			;Restore old palette
		movem.l	d0-d7,$ffff8240.w		;

		lea	save_screenadr,a0		;Restore old screen address
		move.b	(a0)+,$ffff8201.w		;
		move.b	(a0)+,$ffff8203.w		;
		move.b	(a0)+,$ffff820d.w		;

		move.b	#$8,$fffffc02.w			;Enable mouse
		rts



		section	data

image:		incbin	'otter.4pl'			;640*200 four bitplanes with 32 byte palette head
		even



		section	bss

dl_addr:	ds.l	1
displaylist:	ds.l	200
screen:		ds.b	320*200+256	
scraddr:	ds.l	1
vblcount:	ds.w	1
save_pal:	ds.w	16
save_screenadr:	ds.l	1
save_vect_mfp:	ds.b	28+8
save_res:	ds.b	1
save_freq:	ds.b	1
save_hscrl:	ds.b	1
save_lw:	ds.b	1
		even

